library(plyr) # required for rbind.fill()

#define the cohort files, each in a separate csv
cohortfiles <- c( "A" = "taskus a.csv", "B" = "taskus b.csv", "C" = "taskus c.csv", "D" = "taskus d.csv", "E" = "taskus e.csv", "F" = "taskus f.csv", "G" = "taskus g.csv", "H" = "taskus h.csv", "I" = "taskus i.csv")

# this is the start of a big function that processes the data for a cohort
processcohort <- function(cohort){

        # import the data from the file "taskus a.csv" and save a copy that won't be modified
        taskusdata <- read.csv(cohort)

        # rename the columns to make it easier to work with
        colnames(taskusdata) <- c("article", "rating", "adjusted.rating", "notes", "link", "text")

        # load a list of regex patterns for known mirrors and other sites that copy Wikipedia content, aka "badlinks"
        badlinks <- readLines("badlinks.txt")

        # replace every badlink match with "falsepositive"
        for(i in badlinks){
	        taskusdata$link <- sub(i, "falsepositive", taskusdata$link)
        }

        # remove the falsepositive lines that are not the main entry for an article revision (ie, have nothing in the "article" column)
        taskusdata <- taskusdata[taskusdata$article !="" | taskusdata$link != "falsepositive",]

        # if there is only one remaining entry for an article revision, and the link is a falsepositive, then set the adjusted rating to 1 (no plagiarism)

        for(i in 1:(length(taskusdata$article)-1)){
	        if(taskusdata$article[i] != "" & taskusdata$link[i] == "falsepositive" & taskusdata$article[i+1] != "") {
		        taskusdata$adjusted.rating[i] <- 1
	        }
        }
        
        # let's truncate the text field, to cut down on file size so that Google can handle importing the data back again
        taskusdata$text <- factor(strtrim(taskusdata$text, 500))
        
        # remove the excess column that exists in some sheets
        taskusdata <- taskusdata[1:6]
        
        # at this point, we can export taskusdata and put it back into a spreadsheet, and continue adding manually updated ratings and improving the list of badlinks as we go. sending the data back through this script periodically will help speed things along by trimming out cruft.
        write.csv(taskusdata, paste("updated",cohort), row.names=FALSE)
        
        # select all the original taskus ratings that are valid scores, along with the corresponding adjusted ratings
        initialratings <- subset(taskusdata[,2:3], rating %in% c(1,2,3) )

        # substitute in all the adjusted ratings to get "updatedratings"
        updatedratings <- initialratings
        for(i in 1:length(initialratings$rating)){
                if(!is.na(initialratings$adjusted.rating[i])){
                       updatedratings$rating[i] <- initialratings$adjusted.rating[i]
                }
        }

        # remove the excess levels
        updatedratings$rating <- factor(updatedratings$rating)

        # return a column consisting of the updated ratings for the cohort
        return(updatedratings)
}


# use the processcohort function on each cohort file, creating data frames with the names of the cohorts

for(i in 1:length(cohortfiles)){
        assign(names(cohortfiles[i]), processcohort(cohortfiles[i]))
}

# add a column to each cohort filled with the name of the cohort as the value, so that we can then combine all the dataframes into one. try it using assign this time

# there must be a better way to do this, but this will have to do for now
A$cohort <- rep("A", nrow(A))
B$cohort <- rep("B", nrow(B))
C$cohort <- rep("C", nrow(C))
D$cohort <- rep("D", nrow(D))
E$cohort <- rep("E", nrow(E))
F$cohort <- rep("F", nrow(F))
G$cohort <- rep("G", nrow(G))
H$cohort <- rep("H", nrow(H))
I$cohort <- rep("I", nrow(I))

# define the *rows to keep* when we combine the data frames, and then combine them into one data frame
rtk <- c(1,3)
allcohorts <- rbind.fill(A[rtk],B[rtk],C[rtk],D[rtk],E[rtk],F[rtk],G[rtk],H[rtk],I[rtk])

# now let's get ready to plot some data!
library(ggplot2)

# plot the combined data

# ggplot(allcohorts, aes(cohort, fill = rating)) + geom_bar(position = "fill") + scale_fill_brewer(palette="YlOrRd", labels = c("fine", "close paraphrasing", "blatant plagiarism")) + guides(fill = guide_legend(reverse=TRUE)) + theme(legend.background = element_rect(colour = "white"), legend.position = c(.165, .2), panel.background = element_rect(fill = "white")) + ylab("proportion")
# ggplot(allcohorts) + geom_bar(position = "fill", aes(cohort, fill = rating)) + scale_fill_brewer(palette="YlOrRd", labels = c("fine", "close paraphrasing", "blatant plagiarism")) + guides(fill = guide_legend(reverse=TRUE)) + theme(legend.background = element_rect(colour = "white"), legend.position = c(.5, .35), panel.background = element_rect(fill = "white")) + ylab("proportion") + stat_bin(aes(cohort, y = (..count..)/(..count..)/10, label=paste("n =",..count.., sep = '\n')) , geom="text", position = "identity")

ggplot(allcohorts) +
  # the main layer, a bar chart showing the proportion of each rating within each cohort
  layer( geom = "bar", mapping = aes(cohort, fill = rating), position = "fill") +
  # the color palette for the fill, as well as the corresponding legend labels
  scale_fill_brewer(palette="YlOrRd", labels = c("fine", "close paraphrasing", "blatant plagiarism")) +
  # make the legend match the bars, with the plagiarism on top
  guides(fill = guide_legend(reverse=TRUE)) +
  # move the legend into the middle, and the background of both it and the plot itself white
  theme(legend.background = element_rect(colour = "white"), legend.position = c(.5, .35), panel.background = element_rect(fill = "white")) +
  # relabel the y-axis
  ylab("proportion") +
  # add a text layer to show the sample size for each cohort
  layer(geom = "text", stat = "bin", mapping = aes(cohort, y = (..count..)/(..count..)/10, label=paste("n =",..count.., sep = '\n')), position = "identity")


ggsave(file = "all cohorts.svg")

# make a function that plots the data for a cohort and saves it

plotcohort <- function(cohort){

        # create a ggplot object with our rating data
        p <- ggplot(get(cohort), aes(x = rating))

        # make some labels for the different levels of plagiarism, and a colour scheme to go with them
        ratingscale <- c("fine","close paraphrasing","blatant copyvio")
        colourscheme <- c("green", "orange", "red")

        # now use the scale to label the plot object, and apply the colour scheme. NOTE: use factors() to fix the scale problems
        p <- p + scale_x_discrete(labels = ratingscale) + scale_fill_manual(values = colourscheme)

        # make it a histogram
        p <- p + geom_histogram(binwidth = 0.5, origin = -0.25, aes(fill = factor(rating)), colour = "black")

        # generate the plot and print it to a file
        p
        ggsave(file=paste("cohort ",cohort,".svg", sep=""))
}

# # create all the plots using the plotcohort function
# for(cohort in names(cohortfiles)){
#         plotcohort(cohort)
# }

for(i in names(cohortfiles)){
  print(i)
  x <- hist(as.numeric(allcohorts$rating[allcohorts$cohort == i]), breaks =c(0.5,1.5,2.5, 3.5), plot=FALSE)$counts
  print(x)
}