# author: Sage Ross (Wikimedia Foundation)
# license: GPL v2+
# version 1.0
# 2013-09-25

# R script to process and graph data for plagiarism study.
# Place the csv data files and the badlinks.txt file in the working directory before running the script.  
# May require ggplot2 v. 0.9.3.1 or later, and a contemporary version of plyr.

library(plyr) # required for rbind.fill()

#define the cohort files, each in a separate csv
cohortfiles <- c( "A" = "taskus a.csv", "B" = "taskus b.csv", "C" = "taskus c.csv", "D" = "taskus d.csv", "E" = "taskus e.csv", "F" = "taskus f.csv", "G" = "taskus g.csv", "H" = "taskus h.csv", "I" = "taskus i.csv")

# this is the start of a big function that processes the data for a cohort
processcohort <- function(cohort){

        # import the data from the file "taskus a.csv" and save a copy that won't be modified
        taskusdata <- read.csv(cohort)

        # rename the columns to make it easier to work with
        colnames(taskusdata) <- c("article", "rating", "adjusted.rating", "notes", "link", "text")

        # load a list of regex patterns for known mirrors and other sites that copy Wikipedia content, aka "badlinks"
        badlinks <- readLines("badlinks.txt")

        # replace every badlink match with "falsepositive"
        for(i in badlinks){
	        taskusdata$link <- sub(i, "falsepositive", taskusdata$link)
        }

        # remove the falsepositive lines that are not the main entry for an article revision (ie, have nothing in the "article" column)
        taskusdata <- taskusdata[taskusdata$article !="" | taskusdata$link != "falsepositive",]

        # if there is only one remaining entry for an article revision, and the link is a falsepositive, then set the adjusted rating to 1 (no plagiarism)

        for(i in 1:(length(taskusdata$article)-1)){
	        if(taskusdata$article[i] != "" & taskusdata$link[i] == "falsepositive" & taskusdata$article[i+1] != "") {
		        taskusdata$adjusted.rating[i] <- 1
	        }
        }
        
        # let's truncate the text field, to cut down on file size so that Google can handle importing the data back again
        taskusdata$text <- factor(strtrim(taskusdata$text, 500))
        
        # remove the excess column that exists in some sheets
        taskusdata <- taskusdata[1:6]
        
        # at this point, we can export taskusdata and put it back into a spreadsheet, and continue adding manually updated ratings and improving the list of badlinks as we go. sending the data back through this script periodically will help speed things along by trimming out cruft.
        write.csv(taskusdata, paste("updated",cohort), row.names=FALSE)
        
        # select all the original taskus ratings that are valid scores, along with the corresponding adjusted ratings
        initialratings <- subset(taskusdata[,2:3], rating %in% c(1,2,3) )

        # substitute in all the adjusted ratings to get "updatedratings"
        updatedratings <- initialratings
        for(i in 1:length(initialratings$rating)){
                if(!is.na(initialratings$adjusted.rating[i])){
                       updatedratings$rating[i] <- initialratings$adjusted.rating[i]
                }
        }

        # remove the excess levels
        updatedratings$rating <- factor(updatedratings$rating)

        # return a column consisting of the updated ratings for the cohort
        return(updatedratings)
}


# use the processcohort function on each cohort file, creating data frames with the names of the cohorts

for(i in 1:length(cohortfiles)){
        assign(names(cohortfiles[i]), processcohort(cohortfiles[i]))
}

# add a column to each cohort filled with the name of the cohort as the value, so that we can then combine all the dataframes into one.

# there must be a better way to do this, but this will have to do for now
A$cohort <- rep("S new", nrow(A))
B$cohort <- rep("2006", nrow(B))
C$cohort <- rep("2009", nrow(C))
D$cohort <- rep("2012", nrow(D))
E$cohort <- rep("S 2013", nrow(E))
F$cohort <- rep("match", nrow(F))
G$cohort <- rep("S expand", nrow(G))
H$cohort <- rep("active", nrow(H))
I$cohort <- rep("admins", nrow(I))

# define the *rows to keep* when we combine the data frames, and then combine them into one data frame
rtk <- c(1,3)
allcohorts <- rbind.fill(A[rtk],B[rtk],C[rtk],D[rtk],E[rtk],F[rtk],G[rtk],H[rtk],I[rtk])

# define the names and order that we want to show the cohorts in
cohortnames <- c("2006","2009","2012","match","S new","S expand","S 2013","active","admins")

# order the levels
allcohorts$cohort <- factor(allcohorts$cohort, levels = cohortnames )

# now let's get ready to plot some data!
library(ggplot2)
library(RColorBrewer)

# plot the combined data showing the proportion of plagiarism in each cohort
ggplot(allcohorts) +
  # the main layer, a bar chart showing the proportion of each rating within each cohort
  layer( geom = "bar", mapping = aes(cohort, fill = rating), position = "fill") +
  # the color palette for the fill, as well as the corresponding legend labels
  scale_fill_brewer(palette="YlOrRd", labels = c("fine", "close paraphrasing", "blatant plagiarism")) +
  # make the legend match the bars, with the plagiarism on top
  guides(fill = guide_legend(reverse=TRUE)) +
  # move the legend into the middle, and the background of both it and the plot itself white
  theme(legend.background = element_rect(colour = "white"), legend.position = c(.5, .35), panel.background = element_rect(fill = "white")) +
  # relabel the y-axis
  ylab("proportion") +
  # add a text layer to show the sample size for each cohort
  layer(geom = "text", stat = "bin", mapping = aes(cohort, y = (..count..)/(..count..)/10, label=paste("n =",..count.., sep = '\n')), position = "identity")

# save the proportion plot
ggsave(file = "all cohorts by proportion.svg")

# plot the data by percent plagiarism and close paraphrasing instead

# first, reverse the plagiarism levels so that the plagiarism comes first
allcohorts$rating <- factor(allcohorts$rating, levels = c(3,2,1) )

# make a new color pallete that is the reverse of the previous one
colors <- brewer.pal(name="YlOrRd", n=3)
colors <- rev(colors)

ggplot(allcohorts) +
  # the main layer, a bar chart showing the proportion of each rating within each cohort
  layer( geom = "bar", mapping = aes(cohort, fill = rating), position = "fill") +
  # the color palette for the fill, as well as the corresponding legend labels
  scale_fill_manual(values=colors, breaks = c(2,3), labels = c("close paraphrasing", "blatant plagiarism") ) +
  # move the legend into the middle, and the background of both it and the plot itself white
  theme(legend.background = element_rect(colour = "white"), legend.position = c(.8, .85)) +
  # relabel the y-axis, and crop the chart to only show the % plagiarism
  ylab("plagiarism rate") +
  scale_y_continuous(limits=c(0,.15), breaks=c(.025,.05,.075,.1,.125,.15), labels=c("2.5%","5%","7.5%","10%","12.5%","15%")) +
  ## add a text layer to show the sample size for each cohort
  # layer(geom = "text", stat = "bin", mapping = aes(cohort, y = (..count..)/(..count..)/60, label=paste("n =",..count.., sep = '\n')), position = "identity")

# save the percent plot
ggsave(file = "all cohorts by percent.svg")


for(i in cohortnames){
  print(i)
  x <- hist(as.numeric(allcohorts$rating[allcohorts$cohort == i]), breaks =c(0.5,1.5,2.5, 3.5), plot=FALSE)$counts
  print(x)
}
